#include "ball.hh"
#include "explosion.hh"
#include "bat.hh"
#include "bonus.hh"
#include "game.hh"

int direction[] = { +1, 0, 0, +1, -1, 0, 0, -1 };

extern int Brick(int x, int y);
extern Explosions explo;
extern BallContainer balls;
extern Bat bat;
extern Bonuses bonus;
extern double Sin[360], Cos[360];
extern Game game;

BallContainer :: BallContainer()
{
  game.ballsIn = 0;
  for (int t=0;t<MAX_BALLS;t++)
    bv[t].stat=0;
}

BallContainer :: ~BallContainer()
{
}



int BallContainer::Add(int x, int y,int g,int speed,int stat,int bx)
{
  int c = 0;
  while (bv[c].stat != 0 && c<MAX_BALLS)
    c++;
  if (c == MAX_BALLS)
    return -1;
  
  bv[c].x = bv[c].oldx = x;
  bv[c].y = bv[c].oldy = y;
  bv[c].g = g;
  bv[c].speed = speed;
  bv[c].stat = stat;
  bv[c].bx = bx;
  bv[c].ball = 0;
  bv[c].time = 500;
  game.ballsIn++;
  return 0;
}



void BallContainer :: Update()
{
  for (int t=0;t<MAX_BALLS;t++)
    if (bv[t].stat != 0)
      bv[t].Update();
}




void BallContainer :: DeleteAll()
{
  game.ballsIn = 0;
  for (int t=0;t<MAX_BALLS;t++)
    if (bv[t].stat>0)
      {
	//pw_rop((Xv_opaque)pw, bv[t].oldx - 6, bv[t].oldy - 6, 12, 12,
	//       PIX_SRC, poBack.pr, bv[t].oldx - 6, bv[t].oldy - 6);
	bv[t].stat=0;
      }
}


void BallContainer :: ChangeSpeed(int change)
{
  for (int t=0;t<MAX_BALLS;t++)
    bv[t].speed = Limit(bv[t].speed+=change,2,11);
}


void BallContainer :: LaunchAll()
{
  for (int t=0;t<MAX_BALLS;t++)
    if (bv[t].stat == 2)
      {
	bv[t].stat = 1;
	bv[t].g = Limit(int(90 - (bv[t].x - bat.x - bat.w/2)*
			    100/bat.w),15,165);
      }
}


void BallContainer :: ChangeDir()
{
  for (int t=0;t<MAX_BALLS;t++)
    bv[t].g = Limit(bv[t].g+=(random()%20)-10,0,359);
}



Ball :: Ball()
{
  rad = BALL_RADIUS; 
  stat = 1;
}


Ball :: ~Ball()
{

}


Ball& Ball :: operator = (const Ball& ball)
{
  this->rad = ball.rad;
  this->x = ball.x;
  this->y = ball.y;
  this->g = ball.g;
  this->speed = ball.speed;
  this->stat = ball.stat;

  return *this;
}







void Ball::Update()
{
  if (stat == 2)
    {
      x = bat.x + bx;
      y = bat.y - rad;
      Draw();
      time--;
      if (mouseButton == 1 || time<0) 
	{
	  stat = 1;
	  g = Limit(int(90 - (x - bat.x - bat.w/2)*
			100/bat.w),15,165);
	}
    }
 
  if (stat == 1)
    {
      x += Cos[g]*speed;
      y -= Sin[g]*speed;
      
      if ((g > 180) && 
	  (y + rad - Sin[g]*speed> bat.y &&  y-rad + Sin[g]*speed < bat.y) &&
	  (x - rad < bat.x + bat.w && 
	   x + rad > bat.x))
	{
	  
	  if (bat.catchTimes > 0)
	    {
	      bx = int(x - bat.x);
	      stat = 2;
	      time = 500;
	      if (--bat.catchTimes==0)
		{
		  bat.stat = 0;
		  balls.LaunchAll();
		}
	    }
	  else
	    {
	      g = Limit(int(180 - (g - 180) - 
			    (x - bat.x - bat.w/2)*
			    100/bat.w),20,160);
	      if (game.sound==1) audio.Play(15);
	    }
	}
      
      
      if (y > BRICK_HEIGHT * (GAME_HEIGHT-2)) // om bollen ute i y-led
	{
	  stat = 0;
	  game.ballsIn--;
	  pw_rop((Xv_opaque)pw, oldx - 6, oldy - 6, 12, 12,
		 PIX_SRC, poBack.pr, oldx - 6, oldy - 6);
	}
      else
	{
	  BrickBounce();
	  Draw();
	}
    }
} 








void Ball::Draw()
{
  if (oldx != int(x) || oldy != int(y))
    {
      pw_rop((Xv_opaque)pw, oldx - 6, oldy - 6, 12, 12,
	     PIX_SRC, poBack.pr, oldx - 6, oldy - 6);
      
      oldx = int(x);
      oldy = int(y);
    }
  pw_rop((Xv_opaque)pw, oldx - 6, 
	 oldy - 6, 12 , 12, PIX_SRC, poBall.pr, ball*12, 0);
}






void Ball::BrickBounce()
{
  int b;
  if (g<45 || g>314 || (g<225 && g>135))
    {
      b = CheckLR();
      if (b != -1)
	BrickHit(b);
      b = CheckUD();
      if (b != -1)
	BrickHit(b);
    }
  else
    {
      b = CheckUD();
      if (b != -1)
	BrickHit(b);
      b = CheckLR();
      if (b != -1)
	BrickHit(b);
    }
  b = Brick(int(x + Cos[g]*(speed+rad+1)), int(y)); 
  if (brick[b]>0)
    BrickHit(b);
}








int Ball :: CheckLR()
{
  int b = Brick(int(x + Cos[g]*(speed+rad+1)), int(y)); 
  int br = brick[b];
  if (br==0)
    return -1;
  if (brix[br].strength == 0
      || (brix[br].strength < 4 && ball == 1))
    return b;
  if (g < 180)
    {
      g = 180-g;
    }
  else
    {
      g = 180 + (360 - g);
    }
  return b;
}




int Ball :: CheckUD()
{
  int b = Brick(int(x),int(y - Sin[g]*(speed+rad+1))); 
  int br = brick[b];
  if (br==0)
    return -1;
  if (brix[br].strength == 0
      || (brix[br].strength < 4 && ball == 1))
    return b;
  g = 360-g;
  return b;
}





void Ball :: BrickHit(int br)
{
  
  if (br > -1)
    {
      int b = brick[br];
      if(game.sound==1)
	if(brix[b].sound!=-1)
	  audio.Play(brix[b].sound);

      if (brix[b].strength==5 && brix[b].changeTo!=0)
	ChangeBrick(br,brix[b].changeTo);
      else
	if (ball==1)
	  {
	    if (brix[b].strength<4)
	      {
		if (brix[b].changeTo!=0)
		  {
		    int start=b;
		    b=brick[br]=brix[b].changeTo;
		    while(brix[b].changeTo!=0 && b!=start)
		      b=brick[br]=brix[b].changeTo;
		  }
		ChangeBrick(br,brix[b].changeTo);
	      }
	  }
	else
	  if (brix[b].strength<2)
	    {
	      ChangeBrick(br,brix[b].changeTo);
	      if (brix[b].special==0)
		ball = 1;
	    }
    }
}


/*
void Ball :: BrickHit(int br)
{
  
  if (br > -1)
    {
      int b = brick[br];
      if (ball==1)
	{
	  if (brix[b].strength<4)
	    ChangeBrick(br,brix[b].changeTo);
	}
      else
	if (brix[b].strength<2)
	  {
	    ChangeBrick(br,brix[b].changeTo);
	    if (brix[b].special==0)
	      ball = 1;
	  }
    }
}

void ChangeBrick(int br,int type)
{
  int b = brick[br];
  game.score+= brix[b].score;
  
  if (brix[b].bonus!=-1)
    bonus.Add(br%GAME_WIDTH*BRICK_WIDTH,
	      int(br/GAME_WIDTH)*BRICK_HEIGHT,
	      brix[b].bonus);
  
  if (type==0)
    {
      int brx = br%GAME_WIDTH*BRICK_WIDTH;
      int bry = int(br/GAME_WIDTH)*BRICK_HEIGHT;
      RemoveBrick(br);
      explo.Add(brx,bry);
      switch (brix[b].special)
	{
	case 1:                        // take key
	  game.keys--;
	  if (game.keys==0)
	    {
	      brick[game.door]=2;
	      int bry2 = int(game.door/GAME_WIDTH) * BRICK_HEIGHT;
	      int brx2 = game.door%GAME_WIDTH*BRICK_WIDTH;
	      poBack.Op(PIX_SRC).Rect(brx2, bry2,BRICK_WIDTH, BRICK_HEIGHT) = 
		poPict.X(BRICK_WIDTH*2).Y(0);
	      poBack.Op(PIX_SRC&PIX_DST) = poMask.X(BRICK_WIDTH). Y(0);
	      poBack.Op(PIX_SRC|PIX_DST) = poBrick.X(BRICK_WIDTH*2). Y(0);
	      pw_rop((Xv_opaque)pw,brx2,bry2,BRICK_WIDTH,
		     BRICK_HEIGHT, PIX_SRC, poBack.pr,brx2,bry2);
	    }
	  break;
	  
	case 2:                       //  exit reached
	  game.finished = 1;
	  break;
	  
	case 3:                       // more balls
	  balls.Add(brx + BRICK_WIDTH/2, bry + BRICK_HEIGHT/2, random()%360,
		    random()%3+3,1,0);
	  break;
	  
	case 4:                       // smaller bat
	  if (bat.size>0)
	    {
	      balls.LaunchAll();
	      bat.w -= BAT_WIDTH;
	      bat.size--;
	      if (bat.size == 0)
		bat.w = 20;
	      if (bat.size == 1)
		bat.w = 32;
	    }
	  break;
	  
	case 5:                       // higher speed
	  balls.ChangeSpeed(1);
	  break;
	  
	case 6:                       // get mirror
	  SetMouse(GAME_WIDTH*BRICK_WIDTH-mouseX,
		   GAME_HEIGHT/2*BRICK_HEIGHT);
	  bat.mirror = 1 - bat.mirror;
	  break;
	}
    }
  else
    {
      SetBrick(br,type);
    }
}



      int b = brick[br];
      if (b>=UNBREAKABLE && b<SPECIALBREAK)
	{
	  audio.Play(1);
	  return;
	}
      
    
    
      if (ball == 1)
	{
	  if (b==34 || b==35)
	    b=36;
	  if (b>=SPECIALBREAK && b<BREAKABLE)
	    {
	      explo.Add(brx,bry);
	      return;
	    } 
	}
      else
	{
	  if (b>=SPECIALBREAK && b<BREAKABLE)
	    {
	      audio.Play(1);
	      return;
	    } 
	}

      if (b>19 && b !=34 && b !=35 && b<40)
	game.numbricks--;
	

      switch (b)
	{
	case 1:             // take key
	  RemoveBrick(brx,bry);
	  game.keys--;

	  if (game.keys==0)
	    {
	      brick[game.door]=2;
	      int bry2 = game.door/GAME_WIDTH;
	      int brx2 = (game.door - (bry2 * GAME_WIDTH))*BRICK_WIDTH;
	      bry2 = bry2 * BRICK_HEIGHT;
	      poBack.Op(PIX_SRC).Rect(brx2, bry2,BRICK_WIDTH, BRICK_HEIGHT) = 
		poPict.X(BRICK_WIDTH*2).Y(0);
	      poBack.Op(PIX_SRC&PIX_DST) = poMask.X(BRICK_WIDTH). Y(0);
	      poBack.Op(PIX_SRC|PIX_DST) = poBrick.X(BRICK_WIDTH*2). Y(0);
	      pw_rop((Xv_opaque)pw,brx2,bry2,BRICK_WIDTH,
		     BRICK_HEIGHT, PIX_SRC, poBack.pr,brx2,bry2);
	    }
	  break;

	case 2:             //  exit reached
	  game.finished = 1;
	  break;

	case 3:                       // coin
	  RemoveBrick(brx,bry);
	  game.score+=200;
	  break;

     
	case 36:                  // rock
	  audio.Play(2);
	  explo.Add(brx,bry);
	  game.score+=5;
	  break;

	
	
	case 40:                       // more balls
	  explo.Add(brx,bry);
	  balls.Add(brx + BRICK_WIDTH/2, bry + BRICK_HEIGHT/2, g,speed + 1,1,0);
	  game.score+=10;
	  break;

	case 41:                       // get superball
	  explo.Add(brx,bry);
	  ball = 1;
	  break;

	
	case 43:                       // smaller bat
	  if (bat.size>0)
	    {
	      balls.LaunchAll();
	      bat.w -= BAT_WIDTH;
	      bat.size--;
	      if (bat.size == 0)
		bat.w = 20;
	      if (bat.size == 1)
		bat.w = 32;
	    }
	  explo.Add(brx,bry);
	  break;


	case 48:                       // higher speed
	  explo.Add(brx,bry);
	  balls.ChangeSpeed(1);
	  break;

	case 49:                       // get mirror
	  explo.Add(brx,bry);
	  SetMouse(GAME_WIDTH*BRICK_WIDTH-mouseX,
		   GAME_HEIGHT/2*BRICK_HEIGHT);
	  bat.mirror = 1 - bat.mirror;
	  break;

	case 50:
	case 51:
	case 52:
	  explo.Add(brx,bry);
	  break;

	default:
	  CheckBonus(b,brx,bry);
	  break;
	}
    }
}





*/
